/*
    um.c
    Unigram Mixtures, simple implementation.
    $Id: um.c,v 1.4 2013/01/05 06:33:55 daichi Exp $

*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "um.h"
#include "learn.h"
#include "writer.h"
#include "feature.h"
#include "dmatrix.h"
#include "util.h"

int
main (int argc, char *argv[])
{
	int nmixtures  = NMIXTURES_DEFAULT;
	int emmax      = EMMAX_DEFAULT;
	double eta     = ETA_DEFAULT;
	double gamma   = GAMMA_DEFAULT;
	double epsilon = EPSILON_DEFAULT;
	document *data;
	double *lambda, **beta;
	int c, nlex, dlenmax;
	FILE *lp, *bp;

	while ((c = getopt(argc, argv, "M:I:d:e:g:h")) != -1)
	{
		switch (c) {
		case 'M' : nmixtures = atoi(optarg); break;
		case 'I' : emmax = atoi(optarg); break;
		case 'e' : eta = atof(optarg); break;
		case 'g' : gamma = atof(optarg); break;
		case 'd' : epsilon = atof(optarg); break;
		case 'h' : usage (); break;
		default  : break;
		}
	}
	if (argc - optind != 2)
		usage ();

	/* open data */
	if ((data = feature_matrix(argv[optind], &nlex, &dlenmax)) == NULL) {
		fprintf(stderr, "um:: can't open training data.\n");
		exit(1);
	}
	/* allocate parameters */
	if ((lambda  = (double *)calloc(nmixtures, sizeof(double))) == NULL) {
		fprintf(stderr, "um:: can't allocate lambda.\n");
		exit(1);
	}
	if ((beta = dmatrix(nmixtures, nlex)) == NULL) {
		fprintf(stderr, "um:: can't allocate beta.\n");
		exit(1);
	}

	printf("nlex = %d dlenmax = %d\n", nlex, dlenmax);

	/* open output */
	if (((lp = fopen(strconcat(argv[optind + 1], ".lambda"), "w")) == NULL)
	    || ((bp = fopen(strconcat(argv[optind + 1], ".beta"), "w")) == NULL))
	{
		fprintf(stderr, "um:: can't open model outputs.\n");
		exit(1);
	}
	
	/* learn */
	um_learn (data, lambda, beta, nmixtures, nlex,
		  gamma, eta, emmax, epsilon);
	dm_write (lp, bp, lambda, beta, nmixtures, nlex);

	free_feature_matrix (data);
	free_dmatrix (beta, nmixtures);
	free (lambda);

	exit(0);
	
}

void
usage ()
{
	printf(UM_COPYRIGHT);
	printf("$Id: um.c,v 1.4 2013/01/05 06:33:55 daichi Exp $\n");
	printf("usage : %s -M mixtures [-e eta] [-g gamma] [-d epsilon] [-I emmax] train model\n", "um");
	printf("eta     = Dirichlet prior for beta (default %lg)\n", ETA_DEFAULT);
	printf("gamma   = Dirichlet prior for lambda (default %lg)\n", GAMMA_DEFAULT);
	printf("epsilon = relative difference for convergence (default %lg)\n", EPSILON_DEFAULT);
	exit(0);
}

